/*
 * Copyright (C) 2012-2022 Japan Smartphone Security Association
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jssec.android.sharedmemory.inhouseservice.messenger;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.os.SharedMemory;
import android.system.ErrnoException;
import android.widget.Toast;
import android.util.Log;

import org.jssec.android.shared.PkgCert;
import org.jssec.android.shared.SigPerm;
import org.jssec.android.shared.Utils;

import java.nio.ByteBuffer;
import java.nio.ReadOnlyBufferException;

import static android.system.OsConstants.PROT_EXEC;
import static android.system.OsConstants.PROT_READ;
import static android.system.OsConstants.PROT_WRITE;

public class MainActivity extends Activity {

    private final String TAG = "SHMClient";

    // Service へデータを送信するときに使用する Messenger
    private Messenger mServiceMessenger = null;

    // 共有メモリオブジェクト
    private SharedMemory myShared1;
    private SharedMemory myShared2;
    // 共有メモリをマップする ByteBuffer
    private ByteBuffer mBuf1;
    private ByteBuffer mBuf2;

    // 利用先の Activity 情報
    private static final String SHM_PACKAGE =
        "org.jssec.android.sharedmemory.inhouseservice.messenger";
    private static final String SHM_CLASS =
        "org.jssec.android.sharedmemory.inhouseservice.messenger.SHMService";

    // 自社の Signature Permission
    private static final String MY_PERMISSION =
        "org.jssec.android.sharedmemory.inhouseservice.messenger.MY_PERMISSION";

    // 自社の証明書のハッシュ値
    private static String sMyCertHash = null;
    private static String myCertHash(Context context) {
        if (sMyCertHash == null) {
            if (Utils.isDebuggable(context)) {
                // debug.keystore の"androiddebugkey"の証明書ハッシュ値
                sMyCertHash = "0EFB7236 328348A9 89718BAD DF57F544 D5CCB4AE B9DB34BC 1E29DD26 F77C8255";
            } else {
                // keystore の"my company key"の証明書ハッシュ値
                sMyCertHash = "D397D343 A5CBC10F 4EDDEB7C A10062DE 5690984F 1FB9E88B D7B3A7C2 42E142CA";
            }
        }
        return sMyCertHash;
    }

    // Service と接続された状態で true となる
    private boolean mIsBound = false;

    // Service から受け取った Message を処理する Handler
    private class MyHandler extends Handler {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
            case SHMService.SHMEM1:
                // サービスから SHMEM 1 が提供された
                Log.d(TAG, "got SHMEM1");
                // Message.obj に共有メモリが格納されている
                myShared1 = (SharedMemory) msg.obj;
                useSHMEM1();
                break;
            case SHMService.SHMEM2:
                // サービスから SHMEM 2 が提供された
                Log.d(TAG, "got SHMEM2");
                myShared2 = (SharedMemory) msg.obj;
                useSHMEM2();
                break;
            case SHMService.MSG_END:
                Log.d(TAG, "got MSG_END");
                alloverNow();
                break;
            default:
                Log.e(TAG, "invalid message: " + msg.what);
            }
        }
    }

    private Handler mHandler = new MyHandler();

    // Service からデータを受信するときに使用する Messenger
    private Messenger mLocalMessenger = new Messenger(mHandler);

    // Service と接続する時に利用するコネクション。bindService で実装する場合は必要になる
    private class MyServiceConnection implements ServiceConnection {
        // Service に接続された場合に呼ばれる
        public void onServiceConnected(ComponentName className, IBinder service){
            mServiceMessenger = new Messenger(service);
            // 共有メモリ Service にバインドしたタイミングで一つ目の共有メモリを要求する
            sendMessageToService(SHMService.MSG_ATTACH);
        }
        // Service が異常終了して、コネクションが切断された場合に呼ばれる
        public void onServiceDisconnected(ComponentName className){
            mIsBound = false;
            mServiceMessenger = null;
        }
    }
    private MyServiceConnection mServiceConnection;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        doBindService ();
    }

    // 共有メモリ Service に接続する
    private void doBindService () {
        mServiceConnection = new MyServiceConnection();
        if (!mIsBound) {
            // ★ポイント9★ 独自定義Signature Permission が自社アプリにより定義されていることを確認する
            if (!SigPerm.test(this, MY_PERMISSION, myCertHash(this))) {
                Toast.makeText(this,
                              "独自定義Signature Permission が自社アプリにより定義されていない。",
                               Toast.LENGTH_LONG).show();
                return;
            }
            // ★ポイント10★ 利用先アプリの証明書が自社の証明書であることを確認する
            if (!PkgCert.test(this, SHM_PACKAGE, myCertHash(this))) {
                Toast.makeText(this,
                              "利用先サービスは自社アプリではない。",
                               Toast.LENGTH_LONG).show();
                return;
            }
        }
        Intent it = new Intent();
        // ★ポイント11★ 利用先アプリは自社アプリであるから、センシティブな情報を送信してもよい
        it.putExtra("PARAM", "センシティブな情報");

        // ★ポイント12★ 明示的インテントを用いて共有メモリサービスに bind する
        it.setClassName(SHM_PACKAGE, SHM_CLASS);

        if (!bindService(it, mServiceConnection, Context.BIND_AUTO_CREATE)) {
            Toast.makeText(this,
                           "Bind Service Failed", Toast.LENGTH_LONG).show();
            return;
        }
        mIsBound = true;
    }

    // サーバーとの接続を切断する
    private void releaseService () {
        unbindService(mServiceConnection);
    }

    // SHMEM1 を利用する例
    private void useSHMEM1 () {
        // SHMEM1 は読み込みのみが許可されているのでPROT_READ 以外でマップすると例外が発生する
        // 例外は mapMemory() でハンドリングされる
        mBuf1 = mapMemory(myShared1, PROT_WRITE,
                          SHMService.SHMEM1_BUF1_OFFSET, SHMService.SHMEM1_BUF1_LENGTH);
        // PROT_READ でマップ
        mBuf1 = mapMemory(myShared1, PROT_READ,
                          SHMService.SHMEM1_BUF1_OFFSET, SHMService.SHMEM1_BUF1_LENGTH);
        // サービス側がセットしたデータを読み込む
        int len = mBuf1.getInt();
        byte[] bytes = new byte[len];
        mBuf1.get(bytes);
        String message = new String(bytes);
        Toast.makeText(MainActivity.this,
                      "Got: " + message, Toast.LENGTH_LONG).show();
        // read only のため、書き込むと ReadOnlyBufferException 例外が発生する
        try {
            mBuf1.putInt(0);
        } catch (ReadOnlyBufferException e){
            Log.e(TAG, "Write to read only buffer: " + e.getMessage());
        }
        // サービスへリプライ
        sendMessageToService(SHMService.MSG_REPLY1);
        // 続けて書き込み可能な共有メモリをリクエストする
        sendMessageToService(SHMService.MSG_ATTACH2);
    }

    // SHMEM2 を利用する例
    private void useSHMEM2 () {
        // SHMEM2 は write が許可されているため PROT_WRITE でマッピングできる

        // サービス側では PROT_WRITE を設定しているため PROT_READ | PROT_WRITE でマップすると例外が発生する
        mBuf2 = mapMemory(myShared2, PROT_WRITE,
                          SHMService.SHMEM2_BUF1_OFFSET, SHMService.SHMEM2_BUF1_LENGTH);
        if (mBuf2 != null) {
            // しかし PROT_WRITE のみでも read はできる
            int size = mBuf2.getInt();
            byte [] bytes = new byte[size];
            mBuf2.get(bytes);
            String msg = new String(bytes);
            Log.d(TAG, "Got a message from service: " + msg);

            // マップした領域を超えてアクセスすると IndexOutOfBoundsException 例外が発生する
            try {
                mBuf2.get(SHMService.SHMEM2_BUF1_LENGTH + 1);
            } catch (IndexOutOfBoundsException e){
                Log.e(TAG, "out of bound: " + e.getMessage());
            }

            // サービス側で設定したデータを上書きする
            String replyStr = "OK Thanks!";
            mBuf2.putInt(replyStr.length());
            mBuf2.put(replyStr.getBytes());
            // サービスへリプライ
            sendMessageToService(SHMService.MSG_REPLY2);
        }

        if (mBuf2 != null) SharedMemory.unmap(mBuf2);
        // mBuf とオーバーラップする領域を今度は PROT_WRITE | PROT_EXEC でマップ
        // サービス側で共有メモリの PROT_EXEC ビットをオフにしるため、これは失敗する
        // 発生する例外は mapMemory() でハンドリングされている
        mBuf2 = mapMemory(myShared2,PROT_WRITE | PROT_EXEC, 0, 64);

        // マップできる領域サイズはサービス側とは独立している。共有メモリのサイズを超えない限り任意のサイズをマップできる
        mBuf2 = mapMemory(myShared2, PROT_READ | PROT_WRITE,
                          SHMService.SHMEM2_BUF2_OFFSET, SHMService.PAGE_SIZE);
        if (mBuf2 != null) {
            int size = mBuf2.getInt();
            byte [] bytes = new byte[size];
            mBuf2.get(bytes);
            String msg = new String(bytes);
            Log.d(TAG, "Got another message from service: " + msg);
        }
    }
    // 指定の共有メモリをマップする
    private ByteBuffer mapMemory(SharedMemory mem, int proto, int offset, int length){
        ByteBuffer tempBuf;
        try {
            tempBuf = mem.map(proto, offset, length);
        } catch (ErrnoException e){
            Log.e(TAG,
                 "could not map, proto: " + proto + ", offset:" + offset + "," +
                  "length: " + length + "\n " + e.getMessage() + "err no. = " + e.errno);
            return null;
        }
        return tempBuf;
    }

    // サービスへメッセージを送る
    private void sendMessageToService(int what){
        try {
            Message msg = Message.obtain();
            msg.what = what;
            msg.replyTo = mLocalMessenger;
            mServiceMessenger.send(msg);
        } catch (RemoteException e) {
            Log.e(TAG, "Error in sending message: " + e.getMessage());
        }
    }

    // 使い終わった共有メモリを後始末する
    private void alloverNow() {
        // サービスへ使い終わったことを通知
        sendMessageToService(SHMService.MSG_DETACH);
        sendMessageToService(SHMService.MSG_DETACH2);
        // マップされた ByteBuffer を unmap
        if (mBuf1 != null) SharedMemory.unmap(mBuf1);
        if (mBuf2 != null) SharedMemory.unmap(mBuf2);
        // 共有メモリをクローズ
        myShared1.close();
        myShared2.close();
        mBuf1 = null;
        mBuf2 = null;
        myShared1 = null;
        myShared2 = null;
        // サーバーとのコネクションを切る
        releaseService();
    }
}

